@@ -25,8 +25,8 @@
 #' @export
 #' @note The Leiningen engine \code{lein} requires lein-exec plugin; see
 #'   \url{https://github.com/yihui/knitr/issues/1176} for details.
-#' @references Usage: \url{https://yihui.name/knitr/objects/}; examples:
-#'   \url{https://yihui.name/knitr/demo/engines/}
+#' @references Usage: \url{https://yihui.org/knitr/objects/}; examples:
+#'   \url{https://yihui.org/knitr/demo/engines/}
 #' @examples knit_engines$get('python'); knit_engines$get('awk')
 #' names(knit_engines$get())
 knit_engines = new_defaults()
@@ -82,10 +82,10 @@ cache_engines = new_defaults()
 engine_output = function(options, code, out, extra = NULL) {
   if (missing(code) && is.list(out)) return(unlist(wrap(out, options)))
   if (!is.logical(options$echo)) code = code[options$echo]
-  if (length(code) != 1L) code = paste(code, collapse = '\n')
+  if (length(code) != 1L) code = one_string(code)
   if (options$engine == 'sas' && length(out) > 1L && !grepl('[[:alnum:]]', out[2]))
     out = tail(out, -3L)
-  if (length(out) != 1L) out = paste(out, collapse = '\n')
+  if (length(out) != 1L) out = one_string(out)
   out = sub('([^\n]+)$', '\\1\n', out)
   # replace the engine names for markup later, e.g. ```Rscript should be ```r
   options$engine = switch(
@@ -93,27 +93,25 @@ engine_output = function(options, code, out, extra = NULL) {
     options$engine
   )
   if (options$engine == 'stata') {
-    out = gsub('\n\nrunning.*profile.do', '', out)
-    out = sub('...\n\n\n', '', out)
+    out = gsub('\n+running.*profile.do', '', out)
+    out = sub('...\n+', '', out)
     out = sub('\n. \nend of do-file\n', '', out)
   }
-  paste(c(
+  one_string(c(
     if (length(options$echo) > 1L || options$echo) knit_hooks$get('source')(code, options),
     if (options$results != 'hide' && !is_blank(out)) {
       if (options$engine == 'highlight') out else wrap.character(out, options)
     },
     extra
-  ), collapse = '\n')
+  ))
 }
 
 ## command-line tools
 eng_interpreted = function(options) {
   engine = options$engine
   code = if (engine %in% c('highlight', 'Rscript', 'sas', 'haskell', 'stata')) {
-    f = basename(tempfile(engine, '.', switch(
-      engine, sas = '.sas', Rscript = '.R', stata = '.do', '.txt'
-    )))
-    writeLines(c(switch(
+    f = wd_tempfile(engine, switch(engine, sas = '.sas', Rscript = '.R', stata = '.do', '.txt'))
+    write_utf8(c(switch(
       engine,
       sas = "OPTIONS NONUMBER NODATE PAGESIZE = MAX FORMCHAR = '|----|+|---+=|-/<>*' FORMDLIM=' ';title;",
       NULL
@@ -136,15 +134,15 @@ eng_interpreted = function(options) {
           Darwin = paste('-q < %s >', shQuote(xfun::normalize_path(logf))),
           Linux = '-q -e do %s',
           '-q -b do %s'
-         ), shQuote(normalizePath(f)))
+        ), shQuote(normalizePath(f)))
       },
       f
     )
   } else paste(switch(
     engine, bash = '-c', coffee = '-e', groovy = '-e', lein = 'exec -ep',
-    mysql = '-e', node = '-e', octave = '--eval', perl = '-e', psql = '-c',
+    mysql = '-e', node = '-e', octave = '--eval', perl = '-E', psql = '-c',
     python = '-c', ruby = '-e', scala = '-e', sh = '-c', zsh = '-c', NULL
-  ), shQuote(paste(options$code, collapse = '\n')))
+  ), shQuote(one_string(options$code)))
 
   opts = get_engine_opts(options$engine.opts, engine)
   # FIXME: for these engines, the correct order is options + code + file
@@ -152,7 +150,7 @@ eng_interpreted = function(options) {
     paste(code, opts) else paste(opts, code)
   cmd = get_engine_path(options$engine.path, engine)
   out = if (options$eval) {
-    message('running: ', cmd, ' ', code)
+    if (options$message) message('running: ', cmd, ' ', code)
     tryCatch(
       system2(cmd, code, stdout = TRUE, stderr = TRUE, env = options$engine.env),
       error = function(e) {
@@ -162,10 +160,9 @@ eng_interpreted = function(options) {
     )
   } else ''
   # chunk option error=FALSE means we need to signal the error
-  if (!options$error && !is.null(attr(out, 'status')))
-    stop(paste(out, collapse = '\n'))
+  if (!options$error && !is.null(attr(out, 'status'))) stop(one_string(out))
   if (options$eval && engine %in% c('sas', 'stata') && file.exists(logf))
-    out = c(readLines(logf), out)
+    out = c(read_utf8(logf), out)
   engine_output(options, options$code, out)
 }
 
@@ -181,8 +178,8 @@ get_engine_path = function(path, engine) get_engine_opts(path, engine, engine)
 ## C and Fortran (via R CMD SHLIB)
 eng_shlib = function(options) {
   n = switch(options$engine, c = 'c', fortran = 'f', fortran95 = 'f95')
-  f = basename(tempfile(n, '.', paste0('.', n)))
-  writeLines(options$code, f)
+  f = wd_tempfile(n, paste0('.', n))
+  write_utf8(options$code, f)
   on.exit(unlink(c(f, with_ext(f, c('o', 'so', 'dll')))), add = TRUE)
   if (options$eval) {
     out = system(paste('R CMD SHLIB', f), intern = TRUE)
@@ -222,7 +219,7 @@ eng_Rcpp = function(options) {
 
   sourceCpp = getFromNamespace('sourceCpp', 'Rcpp')
 
-  code = paste(options$code, collapse = '\n')
+  code = one_string(options$code)
   # engine.opts is a list of arguments to be passed to Rcpp function, e.g.
   # engine.opts=list(plugin='RcppArmadillo')
   opts = options$engine.opts
@@ -253,7 +250,7 @@ eng_julia = function(options) {
 ## Compiles Stan model in the code chunk, creates a stanmodel object,
 ## and assigns it to a variable with the name given in engine.opts$x.
 eng_stan = function(options) {
-  code = paste(options$code, collapse = '\n')
+  code = one_string(options$code)
   opts = options$engine.opts
   ## name of the modelfit object returned by stan_model
   if (is.null(x <- options$output.var)) {
@@ -280,22 +277,20 @@ eng_stan = function(options) {
 eng_tikz = function(options) {
   if (!options$eval) return(engine_output(options, options$code, ''))
 
-  lines = readLines(tmpl <- options$engine.opts$template %n%
-                      system.file('misc', 'tikz2pdf.tex', package = 'knitr'))
+  lines = read_utf8(options$engine.opts$template %n%
+                    system.file('misc', 'tikz2pdf.tex', package = 'knitr'))
   i = grep('%% TIKZ_CODE %%', lines)
   if (length(i) != 1L)
     stop("Couldn't find replacement string; or the are multiple of them.")
 
   s = append(lines, options$code, i)  # insert tikz into tex-template
-  writeLines(s, texf <- paste0(f <- tempfile('tikz', '.'), '.tex'))
+  write_utf8(s, texf <- wd_tempfile('tikz', '.tex'))
   on.exit(unlink(texf), add = TRUE)
 
   ext = tolower(options$fig.ext %n% dev2ext(options$dev))
 
   to_svg = ext == 'svg'
-  unlink(outf <- paste0(f, if (to_svg) '.dvi' else '.pdf'))
-  tools::texi2dvi(texf, pdf = !to_svg, clean = TRUE)
-  if (!file.exists(outf)) stop('Failed to compile tikz; check the template: ', tmpl)
+  outf = if (to_svg) tinytex::latexmk(texf, 'latex') else tinytex::latexmk(texf)
 
   fig = fig_path(if (to_svg) '.dvi' else '.pdf', options)
   dir.create(dirname(fig), recursive = TRUE, showWarnings = FALSE)
@@ -305,19 +300,15 @@ eng_tikz = function(options) {
   if (to_svg) {
     # dvisvgm needs to be on the path
     # dvisvgm for windows needs ghostscript bin dir on the path also
-    conv = system2('dvisvgm', fig)
-    # copy the svg to figure subdir
-    file.rename(basename(fig2), fig2)
+    if (Sys.which('dvisvgm') == '') tinytex::tlmgr_install('dvisvgm')
+    if (system2('dvisvgm', c('-o', shQuote(fig2), fig)) != 0)
+      stop('Failed to compile ', fig, ' to ', fig2)
   } else {
-    # convert to the desired output-format, calling `convert`
-    conv = 0
-    if (ext != 'pdf') {
-      conv = system2(options$engine.opts[['convert']] %n% 'convert', c(
-        options$engine.opts$convert.opts, sprintf('%s %s', fig, fig2)
-      ))
-    }
+    # convert to the desired output-format using magick
+    if (ext != 'pdf') magick::image_write(do.call(magick::image_convert, c(
+      list(magick::image_read_pdf(fig), ext), options$engine.opts$convert.opts
+    )), fig2)
   }
-  if (conv != 0 && !options$error) stop('Failed to compile ', fig, ' to ', fig2)
   fig = fig2
 
   options$fig.num = 1L; options$fig.cur = 1L
@@ -330,8 +321,8 @@ eng_tikz = function(options) {
 eng_dot = function(options) {
 
   # create temporary file
-  f = tempfile('code', '.')
-  writeLines(code <- options$code, f)
+  f = wd_tempfile('code')
+  write_utf8(code <- options$code, f)
   on.exit(unlink(f), add = TRUE)
 
   # adapt command to either graphviz or asymptote
@@ -386,7 +377,9 @@ eng_cat = function(options) {
     # do not write to stdout like the default behavior of cat()
     if (!identical(file, '')) cat(..., file = file)
   }
-  do.call(cat2, c(list(options$code, sep = '\n'), options$engine.opts))
+  if (options$eval)
+    do.call(cat2, c(list(options$code, sep = '\n'), options$engine.opts))
+
   if (is.null(lang <- options$engine.opts$lang) && is.null(lang <- options$class.source))
     return('')
   options$engine = lang
@@ -395,13 +388,13 @@ eng_cat = function(options) {
 
 ## output the code without processing it
 eng_asis = function(options) {
-  if (options$echo && options$eval) paste(options$code, collapse = '\n')
+  if (options$echo && options$eval) one_string(options$code)
 }
 
 # write a block environment according to the output format
 eng_block = function(options) {
   if (isFALSE(options$echo)) return()
-  code = paste(options$code, collapse = '\n')
+  code = one_string(options$code)
   to = pandoc_to()
   is_pandoc = !is.null(to)
   if (!is_pandoc) {
@@ -443,7 +436,7 @@ eng_block = function(options) {
 eng_block2 = function(options) {
   if (isFALSE(options$echo)) return()
 
-  code = paste(options$code, collapse = '\n'); type = options$type
+  code = one_string(options$code); type = options$type
   if (is.null(type)) return(code)
 
   if (is.null(pandoc_to())) stop('The engine "block2" is for R Markdown only')
@@ -472,8 +465,7 @@ eng_block2 = function(options) {
 eng_html_asset = function(prefix, postfix) {
   function(options) {
     out = if (options$eval && is_html_output(excludes = 'markdown')) {
-      code = c(prefix, options$code, postfix)
-      paste(code, collapse = '\n')
+      one_string(c(prefix, options$code, postfix))
     }
     options$results = 'asis'
     engine_output(options, options$code, out)
@@ -488,7 +480,7 @@ eng_css = eng_html_asset('<style type="text/css">', '</style>')
 
 # perform basic sql parsing to determine if a sql query is an update query
 is_sql_update_query = function(query) {
-  query = paste(query, collapse = '\n')
+  query = one_string(query)
   # remove line comments
   query = gsub('^\\s*--.*\n', '', query)
   # remove multi-line comments
@@ -542,7 +534,7 @@ eng_sql = function(options) {
   max.print = options$max.print %n% (opts_knit$get('sql.max.print') %n% 10)
   if (is.na(max.print) || is.null(max.print))
     max.print = -1
-  sql = paste(options$code, collapse = '\n')
+  sql = one_string(options$code)
 
   query = interpolate_from_env(conn, sql)
   if (isFALSE(options$eval)) return(engine_output(options, query, ''))
@@ -626,8 +618,8 @@ eng_sql = function(options) {
 
 # go engine, added by @hodgesds https://github.com/yihui/knitr/pull/1330
 eng_go = function(options) {
-  f = tempfile('code', '.', fileext = ".go")
-  writeLines(code <- options$code, f)
+  f = wd_tempfile('code', '.go')
+  write_utf8(code <- options$code, f)
   on.exit(unlink(f), add = TRUE)
   cmd = get_engine_path(options$engine.path, options$engine)
 
@@ -658,6 +650,81 @@ eng_go = function(options) {
   engine_output(options, code, extra)
 }
 
+# SASS / SCSS engine (contributed via https://github.com/yihui/knitr/pull/1666)
+#
+# Converts SASS / SCSS -> CSS (with same treatments as CSS engine) using either:
+# LibSass sass R package (https://github.com/rstudio/sass) when
+#   + the package is installed
+#   + engine.opts does not set package = FALSE (e.g. engine.opts = list(package = FALSE))
+#   + an explicit path to the executable is not provided through engine.path, or
+# dart-sass standalone executable (https://sass-lang.com/install) otherwise
+#
+# CSS output is compressed by default but formatting can be set through style in engine.opts
+#  For the sass R package, valid styles are "compressed","expanded", "nested", and "compact"
+#  For the executable, valid styles are "compressed" and "expanded"
+#  Please refer to respective package / executable documentation for more details
+eng_sxss = function(options) {
+
+  # early exit if evaluated output not requested
+  options$results = 'asis'
+  if (!options$eval) return(engine_output(options, options$code, ''))
+
+  # create temporary file with input code
+  f = wd_tempfile('code', paste0('.', options$engine))
+  xfun::write_utf8(options$code , f)
+  on.exit(unlink(f), add = TRUE)
+
+  # process provided engine options
+  package = options$engine.opts$package %n% TRUE
+  style = options$engine.opts$style %n% "compressed"
+  cmd = get_engine_path(options$engine.path, "sass")
+
+  use_package = loadable("sass") && package && cmd == "sass"
+
+  style = match.arg(style, c("compressed", "expanded", if (use_package) c("compact", "nested")))
+  # convert sass/sxss -> css
+  if (use_package) {
+    message("Converting with the R package sass")
+
+    sass_fun = options$engine.opts$sass_fun %n% sass::sass
+    out = tryCatch(
+      sass_fun(sass::sass_file(f), options = sass::sass_options(output_style = style)),
+      error = function(e) {
+        if (!options$error) stop(e)
+        warning2(paste('Error in converting to CSS using sass R package:', e, sep = "\n"))
+        NULL
+      }
+    )
+
+    # remove final newline chars from output
+    if (!is.null(out)) out = sub("\\n$", "", out)
+  } else {
+    message("Converting sass with ", cmd)
+    style = paste0("--style=", style)
+
+    # attempt execution of sass
+    out = tryCatch(
+      system2(cmd, args = c(f, style), stdout = TRUE, stderr = TRUE),
+      error = function(e) {
+        if (!options$error) stop2(e)
+        warning2(paste('Error in converting to CSS using executable:', e, sep = "\n"))
+        NULL
+      }
+    )
+
+    # handle execution errors (status codes)
+    if (!is.null(attr(out, 'status')) && !options$error) stop2(one_string(out))
+  }
+
+  # wrap final output for correct rendering
+  final_out = if (!is.null(out) && is_html_output(excludes = 'markdown')) {
+    one_string(c('<style type="text/css">', out, '</style>'))
+  }
+
+  engine_output(options, options$code, final_out)
+
+}
+
 # set engines for interpreted languages
 local({
   for (i in c(
@@ -673,7 +740,7 @@ knit_engines$set(
   c = eng_shlib, fortran = eng_shlib, fortran95 = eng_shlib, asy = eng_dot,
   cat = eng_cat, asis = eng_asis, stan = eng_stan, block = eng_block,
   block2 = eng_block2, js = eng_js, css = eng_css, sql = eng_sql, go = eng_go,
-  python = eng_python, julia = eng_julia
+  python = eng_python, julia = eng_julia, sass = eng_sxss, scss = eng_sxss
 )
 
 cache_engines$set(python = cache_eng_python)
