@@ -67,7 +67,8 @@
 #' @param envir Environment in which code chunks are to be evaluated,
 #'   for example, \code{\link{parent.frame}()}, \code{\link{new.env}()}, or
 #'   \code{\link{globalenv}()}).
-#' @param encoding Encoding of the input file; see \code{\link{file}}.
+#' @param encoding Encoding of the input file; always assumed to be UTF-8 (i.e.,
+#'   this argument is effectively ignored).
 #' @return The compiled document is written into the output file, and the path
 #'   of the output file is returned. If the \code{text} argument is not
 #'   \code{NULL}, the compiled output is returned as a character vector. In
@@ -87,13 +88,6 @@
 #'   feel confused, print \code{getwd()} in a code chunk to see what the working
 #'   directory really is.
 #'
-#'   The arguments \code{input} and \code{output} do not have to be restricted
-#'   to files; they can be \code{stdin()}/\code{stdout()} or other types of
-#'   connections, but the pattern list to read the input has to be set in
-#'   advance (see \code{\link{pat_rnw}}), and the output hooks should also be
-#'   set (see \code{\link{render_latex}}), otherwise \pkg{knitr} will try to
-#'   guess the patterns and output format.
-#'
 #'   If the \code{output} argument is a file path, it is strongly recommended to
 #'   be in the current working directory (e.g. \file{foo.tex} instead of
 #'   \file{somewhere/foo.tex}), especially when the output has external
@@ -113,9 +107,9 @@
 #'   the R script generated by \code{purl()}. This seems to be obvious, but some
 #'   people \href{http://bit.ly/SnLi6h}{just do not get it}.
 #' @export
-#' @references Package homepage: \url{https://yihui.name/knitr/}. The \pkg{knitr}
-#'   \href{https://yihui.name/knitr/demo/manual/}{main manual}: and
-#'   \href{https://yihui.name/knitr/demo/graphics/}{graphics manual}.
+#' @references Package homepage: \url{https://yihui.org/knitr/}. The \pkg{knitr}
+#'   \href{https://yihui.org/knitr/demo/manual/}{main manual}: and
+#'   \href{https://yihui.org/knitr/demo/graphics/}{graphics manual}.
 #'
 #'   See \code{citation('knitr')} for the citation information.
 #' @examples library(knitr)
@@ -125,23 +119,22 @@
 #' purl(f)  # tangle R code
 #' purl(f, documentation = 0)  # extract R code only
 #' purl(f, documentation = 2)  # also include documentation
-knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE,
-                envir = parent.frame(), encoding = getOption('encoding')) {
+#'
+#' unlink(c('knitr-minimal.tex', 'knitr-minimal.R', 'figure'), recursive = TRUE)
+knit = function(
+  input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE,
+  envir = parent.frame(), encoding = 'UTF-8'
+) {
 
-  # is input from a file? (or a connection on a file)
-  in.file = !missing(input) &&
-    (is.character(input) || prod(inherits(input, c('file', 'connection'), TRUE)))
+  in.file = !missing(input) && is.character(input)  # is input provided?
   oconc = knit_concord$get(); on.exit(knit_concord$set(oconc), add = TRUE)
-  # make a copy of the input path in input2 and change input to file path
-  if (!missing(input)) input2 = input
-  if (in.file && !is.character(input)) input = summary(input)$description
 
   if (child_mode()) {
     setwd(opts_knit$get('output.dir')) # always restore original working dir
     # in child mode, input path needs to be adjusted
     if (in.file && !is_abs_path(input)) {
       input = paste0(opts_knit$get('child.path'), input)
-      input = input2 = file.path(input_dir(TRUE), input)
+      input = file.path(input_dir(TRUE), input)
     }
     # respect the quiet argument in child mode (#741)
     optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
@@ -152,10 +145,9 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     knit_log$restore()
     on.exit(chunk_counter(reset = TRUE), add = TRUE) # restore counter
     adjust_opts_knit()
-    # turn off fancy quotes, use smaller width
+    # turn off fancy quotes, use a null pdf device to record graphics
     oopts = options(
-      useFancyQuotes = FALSE, width = opts_knit$get('width'),
-      knitr.in.progress = TRUE, device = pdf_null
+      useFancyQuotes = FALSE, device = pdf_null, knitr.in.progress = TRUE
     )
     on.exit(options(oopts), add = TRUE)
     # restore chunk options after parent exits
@@ -163,9 +155,7 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     ocode = knit_code$get(); on.exit(knit_code$restore(ocode), add = TRUE)
     on.exit(opts_current$restore(), add = TRUE)
     optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
-    opts_knit$set(tangle = tangle, encoding = encoding,
-                  progress = opts_knit$get('progress') && !quiet
-    )
+    opts_knit$set(tangle = tangle, progress = opts_knit$get('progress') && !quiet)
   }
   # store the evaluation environment and restore on exit
   oenvir = .knitEnv$knit_global; .knitEnv$knit_global = envir
@@ -196,12 +186,13 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     knit_concord$set(infile = input, outfile = output)
   }
 
-  encoding = correct_encode(encoding)
-  text = if (is.null(text)) {
-    readLines(if (is.character(input2)) {
-      con = file(input2, encoding = encoding); on.exit(close(con), add = TRUE); con
-    } else input2, warn = FALSE)
-  } else split_lines(text) # make sure each element is one line
+  if (is.null(text)) {
+    text = readLines(input, encoding = 'UTF-8', warn = FALSE)
+    if (!is_utf8(text)) warning(
+      'The file "', input, '" must be encoded in UTF-8. Please see ',
+      'https://yihui.org/en/2018/11/biggest-regret-knitr/ for more info.'
+    )
+  } else text = split_lines(text) # make sure each element is one line
   if (!length(text)) {
     if (is.character(output)) file.create(output)
     return(output) # a trivial case: create an empty file and exit
@@ -213,14 +204,18 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
     # use ext if cannot auto detect pattern
     if (is.null(pattern <- detect_pattern(text, ext))) {
       # nothing to be executed; just return original input
-      if (is.null(output)) return(paste(text, collapse = '\n')) else {
-        cat(text, sep = '\n', file = output); return(output)
+      if (is.null(output)) {
+        return(if (tangle) '' else one_string(text))
+      } else {
+        write_utf8(if (tangle) '' else text, output)
+        return(output)
       }
     }
-    if (!(pattern %in% names(apat)))
-      stop("a pattern list cannot be automatically found for the file extension '",
-           ext, "' in built-in pattern lists; ",
-           'see ?knit_patterns on how to set up customized patterns')
+    if (!(pattern %in% names(apat))) stop(
+      "a pattern list cannot be automatically found for the file extension '",
+      ext, "' in built-in pattern lists; ",
+      'see ?knit_patterns on how to set up customized patterns'
+    )
     set_pattern(pattern)
     if (pattern == 'rnw' && length(sweave_lines <- which_sweave(text)) > 0)
       remind_sweave(if (in.file) input, sweave_lines)
@@ -256,11 +251,9 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE
   progress = opts_knit$get('progress')
   if (in.file && !quiet) message(ifelse(progress, '\n\n', ''), 'processing file: ', input)
   res = process_file(text, output)
-  res = paste(knit_hooks$get('document')(res), collapse = '\n')
+  res = one_string(knit_hooks$get('document')(res))
   if (tangle) res = c(params, res)
-  if (!is.null(output))
-    writeLines(if (encoding == '') res else native_encode(res, to = encoding),
-               con = output, useBytes = encoding != '')
+  if (!is.null(output)) write_utf8(res, output)
   if (!child_mode()) {
     dep_list$restore()  # empty dependency list
     .knitEnv$labels = NULL
@@ -301,7 +294,7 @@ process_file = function(text, output) {
   wd = getwd()
   for (i in 1:n) {
     if (!is.null(.knitEnv$terminate)) {
-      res[i] = paste(.knitEnv$terminate, collapse = '\n')
+      res[i] = one_string(.knitEnv$terminate)
       knit_exit(NULL)
       break  # must have called knit_exit(), so exit early
     }
@@ -328,7 +321,7 @@ process_file = function(text, output) {
   # output line numbers
   if (concord_mode()) knit_concord$set(outlines = line_count(res))
   print_knitlog()
-  if (tangle) res = res[res != '']
+  if (tangle) res = strip_white(res)
 
   res
 }
@@ -377,7 +370,7 @@ auto_format = function(ext) {
 #' @return A character string of the content of the compiled child document is
 #'   returned as a character string so it can be written back to the parent
 #'   document directly.
-#' @references \url{https://yihui.name/knitr/demo/child/}
+#' @references \url{https://yihui.org/knitr/demo/child/}
 #' @note This function is not supposed be called directly like
 #'   \code{\link{knit}()}; instead it must be placed in a parent document to let
 #'   \code{\link{knit}()} call it indirectly.
@@ -403,9 +396,8 @@ knit_child = function(..., options = NULL, envir = knit_global()) {
       }, add = TRUE)
     }
   }
-  res = knit(..., tangle = opts_knit$get('tangle'), envir = envir,
-             encoding = opts_knit$get('encoding') %n% getOption('encoding'))
-  paste(c('', res), collapse = '\n')
+  res = knit(..., tangle = opts_knit$get('tangle'), envir = envir)
+  one_string(c('', res))
 }
 
 #' Exit knitting early
@@ -549,19 +541,19 @@ wrap.recordedplot = function(x, options) {
       dir.create(dirname(name), recursive = TRUE)
   )
   # vectorize over dev, ext and dpi: save multiple versions of the plot
-  file = mapply(
+  files = mapply(
     save_plot, width = options$fig.width, height = options$fig.height,
     dev = options$dev, ext = options$fig.ext, dpi = options$dpi,
     MoreArgs = list(plot = x, name = name, options = options), SIMPLIFY = FALSE
-  )[[1]]
+  )
+  opts_knit$append(plot_files = unlist(files))
   if (options$fig.show == 'hide') return('')
-  in_base_dir(knit_hooks$get('plot')(file, reduce_plot_opts(options)))
+  in_base_dir(run_hook_plot(files[[1]], reduce_plot_opts(options)))
 }
 
 #' @export
 wrap.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE) {
-  hook_plot = knit_hooks$get('plot')
-  options$fig.num = length(x)
+  if (options$fig.show == 'hide') return('')
   # remove the automatically set out.width when fig.retina is set, otherwise the
   # size of external images embedded via include_graphics() will be set to
   # fig.width * dpi in fix_options()
@@ -572,18 +564,18 @@ wrap.knit_image_paths = function(x, options = opts_chunk$get(), inline = FALSE)
       options['out.width'] = list(NULL)
   }
   dpi = attr(x, 'dpi') %n% options$dpi
+  hook = knit_hooks$get('plot')
   paste(unlist(lapply(seq_along(x), function(i) {
     options$fig.cur = plot_counter()
     if (is.null(options[['out.width']]))
       options['out.width'] = list(raster_dpi_width(x[i], dpi))
-    hook_plot(x[i], reduce_plot_opts(options))
+    hook(x[i], reduce_plot_opts(options))
   })), collapse = '')
 }
 
 #' @export
 wrap.html_screenshot = function(x, options = opts_chunk$get(), inline = FALSE) {
   ext = x$extension
-  hook_plot = knit_hooks$get('plot')
   in_base_dir({
     i = plot_counter()
     if (is.null(f <- x$file)) {
@@ -591,22 +583,30 @@ wrap.html_screenshot = function(x, options = opts_chunk$get(), inline = FALSE) {
       dir.create(dirname(f), recursive = TRUE, showWarnings = FALSE)
       writeBin(x$image, f, useBytes = TRUE)
     }
-    # crop white margins
-    if (isTRUE(options$crop)) in_dir(dirname(f), plot_crop(basename(f)))
     options$fig.cur = i
     options = reduce_plot_opts(options)
     if (!is.null(x$url) && is.null(options$fig.link)) options$fig.link = x$url
-    hook_plot(f, options)
+    run_hook_plot(f, options)
   })
 }
 
+# record plot filenames in opts_knit$get('plot_files'), including those from R
+# code and auto screenshots of HTML widgets, etc. Then run the plot hook.
+run_hook_plot = function(x, options) {
+  opts_knit$append(plot_files = x)
+  hook = knit_hooks$get('plot')
+  hook(x, options)
+}
+
 #' @export
 wrap.knit_embed_url = function(x, options = opts_chunk$get(), inline = FALSE) {
   options$fig.cur = plot_counter()
   options = reduce_plot_opts(options)
+  if (length(extra <- options$out.extra)) extra = paste('', extra, collapse = '')
   add_html_caption(options, sprintf(
-    '<iframe src="%s" width="%s" height="%s"></iframe>',
-    escape_html(x$url), options$out.width %n% '100%', x$height %n% '400px'
+    '<iframe src="%s" width="%s" height="%s"%s></iframe>',
+    escape_html(x$url), options$out.width %n% '100%', x$height %n% '400px',
+    extra %n% ''
   ))
 }
 
@@ -759,7 +759,7 @@ knit_meta_add = function(meta, label = '') {
   if (length(meta)) {
     meta_id = attr(.knitEnv$meta, 'knit_meta_id')
     .knitEnv$meta = c(.knitEnv$meta, meta)
-    attr(.knitEnv$meta, 'knit_meta_id') = c(meta_id, rep(label, length(meta)))
+    attr(.knitEnv$meta, 'knit_meta_id') = c(meta_id, rep_len(label, length(meta)))
   }
   .knitEnv$meta
 }
